/*
 * ProvinceMapBehavior.java
 * Copyright (c) 2008, Lyria.
 * All rights reserved.
 */

package hotel.behavior;

import leon.app.*;
import leon.app.behavior.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * Specific behavior used for the map showing hotels location. In this map, nodes are hotels, they
 * are located depending on their cities coordinates and their representation depends on their
 * category.
 *
 * @author Lyria S.A.
 */
public class ProvinceMapBehavior extends LyMapBehavior
{

	/** Indicates if the map is initialized or not. */
	boolean _initialized = false;

	/**
	 * Updates a node status: the hotel location depending on its city coordinates and the image
	 * representing the hotel depending on its category.
	 *
	 * @param nodeObject the node object corresponding to the hotel
	 * @param mapNode    the node representing the hotel on the map
	 * @param controller current LyMapController instance
	 */
	public void updateNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller)
	{
		// Update the node by the generic method
		super.updateNode(nodeObject, mapNode, controller);

		LyApplication app = controller.getApplication();
		LySession     session = controller.getSession();

		// Gets the hotel city's coordinates
		LyClassInfo   cityClass = app.getClassInfo("city");
		LyObject	  city = nodeObject.getList(session, cityClass).getObject(0);
		LyFieldInfo   xField = cityClass.getFieldInfo("city_x");
		LyFieldInfo   yField = cityClass.getFieldInfo("city_y");
		Integer		  x = (Integer)city.getValue(xField);
		Integer		  y = (Integer)city.getValue(yField);

		// Gets the hotel category
		LyChoiceValue category = nodeObject.getChoiceValue("establishment_category");

		// Set the map node position and image with these values
		mapNode.setX(x.intValue());
		mapNode.setY(y.intValue());
		mapNode.setImage("establishment_" + category.getString() + ".gif");
	}

	/**
	 * The selected hotels have been moved on the map, this method updates their cities coordinates.
	 *
	 * @param controller current mapController instance
	 */
	public void selectionMoved(LyMapController controller)
	{
		// Update the city x and y when a node is moved on the map
		LyApplication app = controller.getApplication();
		LySession     session = controller.getSession();
		LyClassInfo   cityClass = app.getClassInfo("city");
		LyFieldInfo   xField = cityClass.getFieldInfo("city_x");
		LyFieldInfo   yField = cityClass.getFieldInfo("city_y");
		LyObjectList  selection = controller.getSelection();
		LyObject	  establishment, city;
		LyMapNode     node;
		int			  i, n = (selection == null ? 0 : selection.getSize());

		for (i = 0; i < n; i++)
		{
			establishment = selection.getObject(i);
			city = establishment.getList(session, cityClass).getObject(0);
			node = (LyMapNode)controller.getObjectAssoc(establishment);

			if (node != null)
			{
				city.setValue(xField, new Integer((int)node.getX()));
				city.setValue(yField, new Integer((int)node.getY()));
				city.set();
			}
		}
	}

	/**
	 * Complete the map. A geographic layout is used instead of the generic layout (matrix layout),
	 * the viewer is removed, a background is used and an image is added.
	 *
	 * @param controller current mapController instance
	 */
	public void completeMap(LyMapController controller)
	{
		if (!_initialized)
		{
			// Create a geographic layout
			LyMap     map = controller.getMapComponent();
			LyLayout  layout = new LyLayout();
			LyMapNode france = new LyMapNode();

			layout.setStyle(LyLayout.Style.GEOGRAPHIC);
			map.setLayout(layout);

			france.setId("france_node");
			france.setImage("france256.gif");
			france.setStyle(LyMapNode.Style.IMAGE);
			france.setLayer(0);
			france.setResizable(true);
			france.setSensitive(false);

			map.addMapNode(france);
			map.setViewer(true);

			_initialized = true;
		}
	}
} // end class ProvinceMapBehavior
